/*
 * Learn.java
 *
 * Ken Shirriff         ken.shirriff@eng.sun.com
 *
 * Copyright (c) 1997 Sun Microsystems, Inc. All Rights Reserved.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * Please refer to the file "license.txt"
 * for further important copyright and licensing information.
 */

import java.awt.*;
import java.awt.image.*;
import java.applet.Applet;
import java.net.*;
import java.util.*;
import java.io.*;
import pcffont.*;

public class Learn extends Applet {
	PCFFont lfont; // Current font for left entry in lesson
	PCFFont rfont; // Current font for right entry
	PCFFont tfont, bfont; // Current font for top line, rest of lines
	Choice lesson;
	Fontlist fontlist;
	Checkbox direction;
	Button hint;
	Button reset;
	TextField tf;
	int lfonts[] = new int[100], rfonts[] = new int[100];
	String files[] = new String[100];
	int dispWidth, dispHeight;
	int correct = 0;
	int wrong = 0;
	String err_str = null;

	final int h = 31; // Height of the text 
	final int offset = 5;
	final int textoffset = 15;

	// Set up the user interface
	public void init() {
		int numLessons;
		lesson = new Choice();
		lesson.addItem("--Select lesson--");
		setBackground(Color.white);
		String fontlistfile = getParameter("fontlist");
		String lessonfile = getParameter("lessonfile");
		URL url = null;
		DataInputStream dis = null;
		try {
			fontlist = new Fontlist(getDocumentBase(),
			    fontlistfile);
		} catch (Exception e) {
			err("Couldn't open "+fontlistfile);
			return;
		}
		try {
			url = new URL(getDocumentBase(), lessonfile);
			dis = new DataInputStream(url.openStream());
		} catch (Exception e) {
			err("Couldn't open "+lessonfile);
			return;
		}
		try {
			for (numLessons=0; ;numLessons++) {
				String line = dis.readLine();
				if (line == null) {
					break;
				}
				StringTokenizer st = new StringTokenizer(line);
				lesson.addItem(st.nextToken(" "));
				lfonts[numLessons] = Integer.valueOf(st.nextToken(" ")).
				    intValue();
				rfonts[numLessons] = Integer.valueOf(st.nextToken(" ")).
				    intValue();
				files[numLessons] = st.nextToken(" ");
			}
		} catch (Exception e) {
			err("IOException "+e);
			return;
		}

		setLayout(new BorderLayout());
		Panel p = new Panel();
		p.setBackground(Color.white);
		add("South",p);
		p.setLayout(new BorderLayout());
		tf = new TextField("Select a lesson below", 20);
		tf.setEditable(false);
		p.add("North",tf);

		Panel controls = new Panel();
		direction = new Checkbox("Sense");
		controls.add(lesson);
		controls.add(direction);
		hint = new Button("Hint");
		controls.add(hint);
		reset = new Button("Reset");
		controls.add(reset);
		p.add("South",controls);

		validate();
		dispWidth = size().width;
		dispHeight = size().height;


	}

	// Handle a UI event
	public boolean handleEvent(Event e) {
		if (e.target.equals(lesson)) {
			// Hack for Netscape 4.02 problem
			action(e, null);
		}
		return super.handleEvent(e);
	}

	public boolean action(Event e, Object target) {
		if (e.target.equals(lesson)) {
			int idx = lesson.getSelectedIndex();
			if (idx > 0) {
				openLesson(idx-1);
			}
			return true;
		} else if (curLesson>=0 && e.target.equals(reset)) {
			reset();
			newQuestion();
		} else if (curLesson>=0 && e.target.equals(hint)) {
			hint();
			repaint();
		} else if (curLesson>=0 && e.target.equals(direction)) {
			ansl[num_choices+1] = "";
			ansr[num_choices+1] = "";
			updateText();
		}
		return false;
	}

	// Supply a hint
	void hint() {
		int offset = (int)(rand.nextDouble()*num_choices);
		for (int i=0; i<num_choices; i++) {
			int num = (i+offset)%num_choices;
			if (num != right && r_color[num] == null) {
				r_color[num] = Color.gray;
				l_color[num] = Color.gray;
				break;
			}
		}
	}

	void reset() {
		for (int i=0; i<num_choices+2; i++) {
			ansl[i] = "";
			ansr[i] = "";
		}
		correct = 0;
		wrong = 0;
		ansl[num_choices+1] = "";
		ansr[num_choices+1] = "";
		tf.setText("");
	}

	// Handle selection of an answer
	public boolean mouseDown(Event e, int x, int y) {
		if (curLesson < 0) {
			return false;
		}
		int l = y/h-2;
		if (l>=0 && l< num_choices) {
			if (l==right) {
				correct++;
				tf.setText("Correct.  Score: "+correct+"/"+
				    (correct+wrong)+".  Now try this one");
				ansl[num_choices+1] = ansl[right];
				ansr[num_choices+1] = ansr[right];
				l_color[num_choices+1] = Color.gray;
				r_color[num_choices+1] = Color.gray;
				newQuestion();
			} else {
				l_color[l] = Color.gray;
				r_color[l] = Color.gray;
				wrong++;
				tf.setText("Wrong.  Score: "+correct+"/"+
				    (correct+wrong)+".  Try again");
				ansl[num_choices+1] = "";
				ansr[num_choices+1] = "";
				repaint();
			}
		}
		return true;
	}

	public void update(Graphics g) {
		paint(g);
	}

	Vector line1, line2;
	final int num_choices = 5;
	Random rand = new Random();

	int choices[] = new int[num_choices];
	Color l_color[] = new Color[num_choices+2];
	Color r_color[] = new Color[num_choices+2];
	int right;
	String ansl[] = new String[num_choices+2];
	String ansr[] = new String[num_choices+2];

	// Randomly select 5 choices from the list and one that's right.
	void newQuestion() {
		right = (int)(rand.nextDouble()*num_choices);
		int size = line1.size();

		if (size < num_choices) {
			err("File too small!");
			return;
		}
		for (int i=0; i<num_choices; i++) {
			l_color[i] = Color.black;
			r_color[i] = Color.black;
			// Make sure we don't have the element.
			boolean ok = true;
			do {
				ok = true;
				choices[i] = (int)(rand.nextDouble()*size);
				for (int j=0; j<i; j++) {
					if (choices[i] == choices[j]) {
						ok = false;
						break;
					}
					if (
					  ((String)line1.elementAt(choices[i])).equals((String)line1.elementAt(choices[j])) ||
					  ((String)line2.elementAt(choices[i])).equals((String)line2.elementAt(choices[j]))) {
						ok = false;
						break;
					}
				}
			} while (!ok);
		}
		updateText();
	}

	void updateText() {
		Vector q, a;
		if (direction.getState()) {
			q = line1;
			a = line2;
			tfont = lfont;
			bfont = rfont;
		} else {
			q = line2;
			a = line1;
			tfont = rfont;
			bfont = lfont;
		}

		for (int i=0; i<num_choices; i++) {
			ansl[i] = (String)a.elementAt(choices[i]);
			ansr[i] = (String)q.elementAt(choices[i]);
			l_color[i] = Color.black;
			r_color[i] = Color.white;
		}
		repaint();
	}

	int curLesson = -1;

	// Read in a lesson file
	void openLesson(int num) {
		if (num == curLesson) {
			return;
		}
		tf.setText("Loading...");
		line1 = new Vector();
		line2 = new Vector();
		try {
			lfont = fontlist.openfont(lfonts[num]);
		} catch (Exception e) {
			err("Couldn't open font "+lfonts[num]);
			return;
		}
		try {
			rfont = fontlist.openfont(rfonts[num]);
		} catch (Exception e) {
			err("Couldn't open font "+rfonts[num]);
			return;
		}
		try {
			URL url = new URL(getDocumentBase(), files[num]);
			DataInputStream is = new DataInputStream(
			    url.openStream());

			while (true) {
				String line = is.readLine();
				if (line == null) {
					break;
				}
				int p = line.indexOf('=');
				String l, r;
				l = line.substring(0, p);
				r = line.substring(p+1);
				if (lfont.is2Byte()) {
					l = PCFFont.toJISX(l);
				} else {
					l = PCFFont.toUnicode(l);
				}
				if (rfont.is2Byte()) {
					r = PCFFont.toJISX(r);
				}
				line1.addElement(l);
				line2.addElement(r);
			};
		} catch (Exception e) {
			tf.setText("Error loading lesson");
			err("Exception opening lesson "+e);
			return;
		}
		curLesson = num;
		reset();
		newQuestion();
		tf.setText("Click on the matching answer");
	}

	// Redraw the screen
	public void paint(Graphics g) {
		if (err_str != null) {
			// Display an error message
			g.setColor(Color.blue);
			g.fillRect(0, 0, dispWidth, dispHeight);
			g.setColor(Color.black);
			g.drawString(err_str, 10, 20);
			return;
		}
		g.setColor(Color.black);
		// Draw the boxes
		g.drawRect(0, 0, dispWidth-1, dispHeight-1);
		g.drawRect(offset,offset,dispWidth-2*offset,h);
		for (int i=0; i<num_choices; i++) {
			g.setColor(Color.lightGray);
			g.drawRect(offset,offset+h*(i+2),
			    dispWidth-2*offset,h-6);
			g.setColor(Color.black);
			g.drawRect(offset+1,offset+h*(i+2)+1,
			    dispWidth-2*offset,h-6);
		}
		if (tfont != null) {
			// Draw the strings
			g.clipRect(offset,0, dispWidth-2*offset-6,dispHeight);
			int tFontOffset = (h+tfont.getAscent()-tfont.getDescent())/2+offset;
			int bFontOffset = (h+bfont.getAscent()-bfont.getDescent())/2+offset;
			tfont.setFgColor(Color.black);
			tfont.setBgColor(Color.white);
			bfont.setBgColor(Color.white);
			g.clearRect(offset+1, offset+1, dispWidth-2*offset-2,
			    h-2);
			tfont.drawString(g, ansr[right], textoffset, tFontOffset);
			for (int i=0; i<num_choices+2; i++) {
				g.clearRect(offset+2, offset+h*(i+2)+2,
				    dispWidth-2*offset-6,h-8);
				bfont.setFgColor(l_color[i]);
				bfont.drawString(g, ansl[i], textoffset,
				    h*(i+2)+bFontOffset);
				tfont.setFgColor(r_color[i]);
				tfont.drawString(g, ansr[i], textoffset+
				    bfont.stringWidth(ansl[i])+10,
				    h*(i+2)+bFontOffset);
			}
		}
	}

        public String getAppletInfo() {
                return "Learn by Ken Shirriff  shirriff@eng.sun.com";
        }

	// Display an error;
	void err(String s) {
		err_str = s;
		repaint();
		showStatus(s);
		System.out.println("Error: "+s);
		if (tf != null) {
			tf.setText(s);
		}
	}
}
